@@ -2,6 +2,7 @@ from sqlalchemy import Column, String, ForeignKey, Float, Integer
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import relationship, backref
 from sqlalchemy.schema import PrimaryKeyConstraint
+
 import sys
 
 # needed to avoid circular imports
@@ -17,9 +18,9 @@ do the lifiting.
 """
 
 import re
-from collections import OrderedDict
 
 def _ncbi_parse(html):
+    from collections import OrderedDict
 
     try:
         info = html.split("Sequences producing significant alignments")[1].split("<tbody>")[1]
@@ -108,10 +109,30 @@ class ABase(object):
     def exons(self):
         # drop the trailing comma
         if not self.is_gene_pred: return []
-        starts = (long(s) for s in self.exonStarts[:-1].split(","))
-        ends = (long(s) for s in self.exonEnds[:-1].split(","))
+        if hasattr(self, "exonStarts"):
+            starts = (long(s) for s in self.exonStarts[:-1].split(","))
+            ends = (long(s) for s in self.exonEnds[:-1].split(","))
+        else: # it is bed12
+            starts = [self.start + long(s) for s in self.chromStarts[:-1].split(",")]
+            ends = [starts[i] + long(size) for i, size \
+                    in enumerate(self.blockSizes[:-1].split(","))]
+
+
         return zip(starts, ends)
 
+    def tss(self, up=0, down=0):
+        if not self.is_gene_pred: return None
+        tss = self.txEnd if self.strand == '-' else self.txStart
+        start, end = tss, tss
+        if self.strand == '+':
+            start -= up
+            end += down
+        else:
+            start += up
+            end -= down
+            start, end = end, start
+        return start, end
+
     @property
     def coding_exons(self):
         """
@@ -211,18 +232,24 @@ class ABase(object):
         return e.g. "intron;exon" if the other_start, end overlap introns and
         exons
         """
+        # completely encases gene.
+        if other_start <= self.start and other_end >= self.end:
+            return ['gene' if self.cdsStart != self.cdsEnd else 'nc_gene']
         other = Interval(other_start, other_end)
         ovls = []
         tx = 'txEnd' if self.strand == "-" else 'txStart'
-        if hasattr(self, tx) and other_start <= self.txStart <= other_end and self.txStart != self.txEnd:
+        if hasattr(self, tx) and other_start <= getattr(self, tx) <= other_end \
+            and self.cdsStart != self.cdsEnd:
                 ovls = ["TSS"]
-        # TODO check txStart == txEnd and return non-coding?
         for ftype in ('introns', 'exons', 'utr5', 'utr3', 'cdss'):
             feats = getattr(self, ftype)
             if not isinstance(feats, list): feats = [feats]
             if any(Interval(f[0], f[1]).overlaps(other) for f in feats):
                 ovls.append(ftype[:-1] if ftype[-1] == 's' else ftype)
-        if 'cds' in ovls: return [ft for ft in ovls if ft != 'exon']
+        if 'cds' in ovls:
+            ovls = [ft for ft in ovls if ft != 'exon']
+        if self.cdsStart == self.cdsEnd:
+            ovls = ['nc_' + ft for ft in ovls]
         return ovls
 
     def distance(self, other_or_start=None, end=None, features=False):
@@ -320,7 +347,7 @@ class ABase(object):
     def db(self):
         # grab the database name from the current row
         # e.g. hg18
-        return self.metadata.bind.url.database
+        return self._table.bind.url.database
 
     def __str__(self):
         # output something bed-like
@@ -415,7 +442,7 @@ class ABase(object):
         """
         http://genome.ucsc.edu/FAQ/FAQformat.html#format9
         """
-        return hasattr(self, "exonStarts")
+        return hasattr(self, "exonStarts") or hasattr(self, 'chromStarts')
 
     def bed(self, *attrs, **kwargs):
         exclude = ("chrom", "start", "end", "txStart", "txEnd", "chromStart",
@@ -502,7 +529,7 @@ def get_start_end(other_or_start, end):
     return other_start, other_end
 
 class Feature(ABase):
-    name = Column(String, unique=True, primary_key=True)
+    name = Column(String, unique=False, primary_key=True)
 
 class cpgIslandExt(Feature):
     anno_cols = ("name", "distance", "feature")
@@ -547,14 +574,6 @@ class chromInfo(ABase):
 
     __str__ = __repr__
 
-class kgXref(ABase):
-    __tablename__ = "kgXref"
-    kgID = Column(String, primary_key=True)
-
-    def __repr__(self):
-        return "%s(%s/%s)" % (self.__tablename__, self.geneSymbol, self.kgID)
-
-    __str__ = __repr__
 
 
 class Blat(Feature):
@@ -576,20 +595,43 @@ class Blat(Feature):
     def hit_length(self):
         return self.span
 
+class kgXref(ABase):
+    __tablename__ = "kgXref"
+
+    kgID = Column(String, primary_key=True)
+
+#    @declared_attr
+#    def kgID(self):
+#        return Column(String, ForeignKey('knownGene.name'), primary_key=True)
+
+
+    def __repr__(self):
+        return "%s(%s/%s)" % (self.__tablename__, self.geneSymbol, self.kgID)
+
+    __str__ = __repr__
+
 class knownGene(ABase):
     __tablename__ = "knownGene"
 
+    __mapper_args__= {'always_refresh': False, 'exclude_properties': ['dist',
+        '_dist']}
+
     __preload_classes__ = ("kgXref",)
 
     anno_cols = ("name", "distance", "feature")
 
     @declared_attr
     def name(cls):
-        return Column(String, ForeignKey('kgXref.kgID'), primary_key=True)
+        return Column(ForeignKey('kgXref.kgID'), primary_key=True)
 
-    @declared_attr
-    def kgXref(cls):
-        return relationship("kgXref", backref=backref("knownGene"), lazy="subquery")
+    #@declared_attr
+    #def kgXref(cls):
+    #    #return relationship("kgXref", backref=backref("knownGene"), lazy="subquery")
+#
+#        return relationship(lambda: kgXref,
+#                primaryjoin=lambda: knownGene.name==kgXref.kgID,
+#            lazy="subquery")
+            #viewonly=True)
 
     @property
     def name2(self):
